/**
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2013-2013 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Oracle. Portions Copyright 2013-2013 Oracle. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.html.json.impl;

import java.util.Map;
import net.java.html.BrwsrCtx;
import net.java.html.json.MapModelTest;
import net.java.html.json.Model;
import net.java.html.json.Models;
import net.java.html.json.OnPropertyChange;
import net.java.html.json.Property;
import org.apidesign.html.context.spi.Contexts;
import org.apidesign.html.json.spi.Technology;
import static org.testng.Assert.*;
import org.testng.annotations.Test;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
public class PlainEnumTest {
    @Test public void unionA() {
        Union on = new Union(new Union.A(9), 11, 1.1);
        assertEquals(on.getX(), 11);
        assertEquals(on.getY(), 1.1);
        
        assertEquals(on.getAbc(), Abc.A);
        assertNotNull(on.getA());
        assertEquals(on.getA().getUa().size(), 1);
        assertEquals(on.getA().getUa().get(0), Integer.valueOf(9));
        assertNull(on.getB());
    }

    @Test public void unionB() {
        Union on = new Union(new Union.B("9.9"), 11, 1.1);
        assertEquals(on.getX(), 11);
        assertEquals(on.getY(), 1.1);
        
        assertEquals(on.getAbc(), Abc.B);
        assertNull(on.getA());
        assertNotNull(on.getB());
        assertEquals(on.getB().getUb(), "9.9");
    }

    @Test public void cloneUnionB() {
        Union old = new Union(new Union.B("9.9"), 11, 1.1);
        Union on = old.clone();
        assertNotSame(old.getB(), on.getB());
        
        assertEquals(on.getX(), 11);
        assertEquals(on.getY(), 1.1);
        
        assertEquals(on.getAbc(), Abc.B);
        assertNull(on.getA());
        assertNotNull(on.getB());
        assertEquals(on.getB().getUb(), "9.9");
    }

    @Model(className = "Union", properties = {
        @Property(name = "x", type = int.class),
        @Property(name = "y", type = double.class)
    })
    enum Abc {
        @Model(className = "A", properties = { 
            @Property(name = "ua", type = int.class, array = true)
        })
        A, 
        @Model(className = "B", properties = { 
            @Property(name = "ub", type = String.class)
        })
        B;
        
        @OnPropertyChange({ "ua", "ub" })
        static void onChange(Union u) {
            u.setX(u.getX() + 1);
        }
    }
    
    @Model(className = "EnumAndModel", properties = {
        @Property(name = "enm", type = Abc.class),
        @Property(name = "mdl", type = Union.class),
    })
    static class EnumAndModelModel {
    }
    
    @Test public void enumIsTreatedAsEnum() {
        EnumAndModel em = new EnumAndModel();
        assertNull(em.getEnm(), "No enum set, yet");
        assertNotNull(em.getMdl(), "Model is not null");
    }
    
    @Test public void serializeAsASingleObject() {
        Union on = new Union(new Union.B("9.9"), 11, 1.1);
        String json = on.toString();
        assertNotEquals(json.indexOf("\"9.9\""), -1, "The string 9.9 should be in the output");
        assertNotEquals(json.indexOf("Abc"), -1, "The property 'Abc' should be in");
        assertNotEquals(json.indexOf("\"Abc\""), -1, "The string 'Abc' should be assigned");
        
        int first = json.indexOf('{');
        assertEquals(first, 0, "Begins with {");
        
        int next = json.indexOf('{', 1);
        assertEquals(next, -1, "No other {");
    }
    
    @Test public void unionOfProperties() {
        MapModelTest.MapTechnology mt = new MapModelTest.MapTechnology();
        BrwsrCtx ctx = Contexts.newBuilder().register(Technology.class, mt, 1).build();

        Union on = new Union(new Union.B("9.9"), 11, 1.1);
        Union u = Models.bind(on, ctx);
        
        Map<?,?> map = (Map<?,?>)Models.toRaw(u);
        assertNotNull(map.get("x"), "Four properties: " + map);
        assertNotNull(map.get("y"), "Four properties: " + map);
        MapModelTest.One ub;
        assertNotNull(map.get("ua"), "Four properties: " + map);
        assertNotNull(ub = (MapModelTest.One) map.get("ub"), "Four properties: " + map);
        
        assertEquals(u.getX(), 11, "Eleven");
        
        assertEquals(ub.getChanges(), 0, "No changes yet");
        u.getB().setUb("10");
        assertEquals(u.getX(), 12, "Change increases X");
        assertEquals(ub.getChanges(), 1, "One change");
    }
    
    @Test public void changeInUnionArray() {
        MapModelTest.MapTechnology mt = new MapModelTest.MapTechnology();
        BrwsrCtx ctx = Contexts.newBuilder().register(Technology.class, mt, 1).build();

        Union on = new Union(new Union.A(10, 20, 30), 0, 1.1);
        Union u = Models.bind(on, ctx);

        Map<?, ?> map = (Map<?, ?>) Models.toRaw(u);
        MapModelTest.One ua;
        assertNotNull(ua = (MapModelTest.One) map.get("ua"), "Array prop found: " + map);
        
        assertEquals(u.getX(), 0, "Zero");
        assertEquals(ua.getChanges(), 0, "No changes yet");
        u.getA().getUa().add(40);
        assertEquals(u.getX(), 1, "Incremented once");
        assertEquals(ua.getChanges(), 1, "One change");
    }
}
